from __future__ import absolute_import

from tf_pose import network_base
import tensorflow as tf


class CmuNetwork(network_base.BaseNetwork):
    def setup(self):
        (
            self.feed("image")
            .normalize_vgg(name="preprocess")
            .conv(3, 3, 64, 1, 1, name="conv1_1")
            .conv(3, 3, 64, 1, 1, name="conv1_2")
            .max_pool(2, 2, 2, 2, name="pool1_stage1", padding="VALID")
            .conv(3, 3, 128, 1, 1, name="conv2_1")
            .conv(3, 3, 128, 1, 1, name="conv2_2")
            .max_pool(2, 2, 2, 2, name="pool2_stage1", padding="VALID")
            .conv(3, 3, 256, 1, 1, name="conv3_1")
            .conv(3, 3, 256, 1, 1, name="conv3_2")
            .conv(3, 3, 256, 1, 1, name="conv3_3")
            .conv(3, 3, 256, 1, 1, name="conv3_4")
            .max_pool(2, 2, 2, 2, name="pool3_stage1", padding="VALID")
            .conv(3, 3, 512, 1, 1, name="conv4_1")
            .conv(3, 3, 512, 1, 1, name="conv4_2")
            .conv(3, 3, 256, 1, 1, name="conv4_3_CPM")
            .conv(3, 3, 128, 1, 1, name="conv4_4_CPM")  # *****
            .conv(3, 3, 128, 1, 1, name="conv5_1_CPM_L1")
            .conv(3, 3, 128, 1, 1, name="conv5_2_CPM_L1")
            .conv(3, 3, 128, 1, 1, name="conv5_3_CPM_L1")
            .conv(1, 1, 512, 1, 1, name="conv5_4_CPM_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="conv5_5_CPM_L1")
        )

        (
            self.feed("conv4_4_CPM")
            .conv(3, 3, 128, 1, 1, name="conv5_1_CPM_L2")
            .conv(3, 3, 128, 1, 1, name="conv5_2_CPM_L2")
            .conv(3, 3, 128, 1, 1, name="conv5_3_CPM_L2")
            .conv(1, 1, 512, 1, 1, name="conv5_4_CPM_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="conv5_5_CPM_L2")
        )

        (
            self.feed("conv5_5_CPM_L1", "conv5_5_CPM_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage2")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage2_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage2_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage2_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage2_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage2_L1")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage2_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage2_L1")
        )

        (
            self.feed("concat_stage2")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage2_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage2_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage2_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage2_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage2_L2")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage2_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage2_L2")
        )

        (
            self.feed("Mconv7_stage2_L1", "Mconv7_stage2_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage3")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage3_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage3_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage3_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage3_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage3_L1")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage3_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage3_L1")
        )

        (
            self.feed("concat_stage3")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage3_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage3_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage3_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage3_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage3_L2")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage3_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage3_L2")
        )

        (
            self.feed("Mconv7_stage3_L1", "Mconv7_stage3_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage4")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage4_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage4_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage4_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage4_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage4_L1")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage4_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage4_L1")
        )

        (
            self.feed("concat_stage4")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage4_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage4_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage4_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage4_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage4_L2")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage4_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage4_L2")
        )

        (
            self.feed("Mconv7_stage4_L1", "Mconv7_stage4_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage5")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage5_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage5_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage5_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage5_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage5_L1")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage5_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage5_L1")
        )

        (
            self.feed("concat_stage5")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage5_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage5_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage5_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage5_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage5_L2")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage5_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage5_L2")
        )

        (
            self.feed("Mconv7_stage5_L1", "Mconv7_stage5_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage6")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage6_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage6_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage6_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage6_L1")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage6_L1")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage6_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage6_L1")
        )

        (
            self.feed("concat_stage6")
            .conv(7, 7, 128, 1, 1, name="Mconv1_stage6_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv2_stage6_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv3_stage6_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv4_stage6_L2")
            .conv(7, 7, 128, 1, 1, name="Mconv5_stage6_L2")
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage6_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage6_L2")
        )

        with tf.variable_scope("Openpose"):
            (
                self.feed("Mconv7_stage6_L2", "Mconv7_stage6_L1").concat(
                    3, name="concat_stage7"
                )
            )

    def loss_l1_l2(self):
        l1s = []
        l2s = []
        for layer_name in self.layers.keys():
            if "Mconv7" in layer_name and "_L1" in layer_name:
                l1s.append(self.layers[layer_name])
            if "Mconv7" in layer_name and "_L2" in layer_name:
                l2s.append(self.layers[layer_name])

        return l1s, l2s

    def loss_last(self):
        return self.get_output("Mconv7_stage6_L1"), self.get_output("Mconv7_stage6_L2")

    def restorable_variables(self):
        return None



import numpy as np
import math


def cropRect(rect, cropTop, cropBottom, cropLeft, cropRight):
    """
	Crops a rectangle by the specified number of pixels on each side.
	
	The input rectangle and return value are both a tuple of (x,y,w,h).
	"""

    # Unpack the rectangle
    x, y, w, h = rect

    # Crop by the specified value
    x += cropLeft
    y += cropTop
    w -= cropLeft + cropRight
    h -= cropTop + cropBottom

    # Re-pack the padded rect
    return (x, y, w, h)


def padRect(rect, padTop, padBottom, padLeft, padRight, bounds, clipExcess=True):
    """
	Pads a rectangle by the specified values on each individual side,
	ensuring the padded rectangle falls within the specified bounds.
	
	The input rectangle, bounds, and return value are all a tuple of (x,y,w,h).
	"""

    # Unpack the rectangle
    x, y, w, h = rect

    # Pad by the specified value
    x -= padLeft
    y -= padTop
    w += padLeft + padRight
    h += padTop + padBottom

    # Determine if we are clipping overflows/underflows or
    # shifting the centre of the rectangle to compensate
    if clipExcess == True:

        # Clip any underflows
        x = max(0, x)
        y = max(0, y)

        # Clip any overflows
        overflowY = max(0, (y + h) - bounds[0])
        overflowX = max(0, (x + w) - bounds[1])
        h -= overflowY
        w -= overflowX

    else:

        # Compensate for any underflows
        underflowX = max(0, 0 - x)
        underflowY = max(0, 0 - y)
        x += underflowX
        y += underflowY

        # Compensate for any overflows
        overflowY = max(0, (y + h) - bounds[0])
        overflowX = max(0, (x + w) - bounds[1])
        x -= overflowX
        w += overflowX
        y -= overflowY
        h += overflowY

        # If there are still overflows or underflows after our
        # modifications, we have no choice but to clip them
        x, y, w, h = padRect((x, y, w, h), 0, 0, 0, 0, bounds, True)

    # Re-pack the padded rect
    return (x, y, w, h)


def cropRectEqually(rect, cropping):
    """
	Crops a rectangle by the specified number of pixels on all sides.
	
	The input rectangle and return value are both a tuple of (x,y,w,h).
	"""
    return cropRect(rect, cropping, cropping, cropping, cropping)


def padRectEqually(rect, padding, bounds, clipExcess=True):
    """
	Applies equal padding to all sides of a rectangle,
	ensuring the padded rectangle falls within the specified bounds.
	
	The input rectangle, bounds, and return value are all a tuple of (x,y,w,h).
	"""
    return padRect(rect, padding, padding, padding, padding, bounds, clipExcess)


def squareAspect(rect):
    """
	Crops either the width or height, as necessary, to make a rectangle into a square.
	
	The input rectangle and return value are both a tuple of (x,y,w,h).
	"""

    # Determine which dimension needs to be cropped
    x, y, w, h = rect
    if w > h:
        cropX = (w - h) // 2
        return cropRect(rect, 0, 0, cropX, cropX)
    elif w < h:
        cropY = (h - w) // 2
        return cropRect(rect, cropY, cropY, 0, 0)

    # Already a square
    return rect


def fitToSize(rect, targetWidth, targetHeight, bounds):
    """
	Pads or crops a rectangle as necessary to achieve the target dimensions,
	ensuring the modified rectangle falls within the specified bounds.
	
	The input rectangle, bounds, and return value are all a tuple of (x,y,w,h).
	"""

    # Determine the difference between the current size and target size
    x, y, w, h = rect
    diffX = w - targetWidth
    diffY = h - targetHeight

    # Determine if we are cropping or padding the width
    if diffX > 0:
        cropLeft = math.floor(diffX / 2)
        cropRight = diffX - cropLeft
        x, y, w, h = cropRect((x, y, w, h), 0, 0, cropLeft, cropRight)
    elif diffX < 0:
        padLeft = math.floor(abs(diffX) / 2)
        padRight = abs(diffX) - padLeft
        x, y, w, h = padRect((x, y, w, h), 0, 0, padLeft, padRight, bounds, False)

    # Determine if we are cropping or padding the height
    if diffY > 0:
        cropTop = math.floor(diffY / 2)
        cropBottom = diffY - cropTop
        x, y, w, h = cropRect((x, y, w, h), cropTop, cropBottom, 0, 0)
    elif diffY < 0:
        padTop = math.floor(abs(diffY) / 2)
        padBottom = abs(diffY) - padTop
        x, y, w, h = padRect((x, y, w, h), padTop, padBottom, 0, 0, bounds, False)

    return (x, y, w, h)



from .ArrayUtils import *
import numpy as np
import math


def generateDistanceMatrix(width, height):
    """
	Generates a matrix specifying the distance of each point in a window to its centre.
	"""

    # Determine the coordinates of the exact centre of the window
    originX = width / 2
    originY = height / 2

    # Generate the distance matrix
    distances = zerosFactory((height, width), dtype=np.float)
    for index, val in np.ndenumerate(distances):
        y, x = index
        distances[(y, x)] = math.sqrt(
            math.pow(x - originX, 2) + math.pow(y - originY, 2)
        )

    return distances



import os
from os.path import dirname, abspath

import tensorflow as tf

from tf_pose.network_mobilenet import MobilenetNetwork
from tf_pose.network_mobilenet_thin import MobilenetNetworkThin

from tf_pose.network_cmu import CmuNetwork
from tf_pose.network_mobilenet_v2 import Mobilenetv2Network


def _get_base_path():
    if not os.environ.get("OPENPOSE_MODEL", ""):
        return "./models"
    return os.environ.get("OPENPOSE_MODEL")


def get_network(type, placeholder_input, sess_for_load=None, trainable=True):
    if type == "mobilenet":
        net = MobilenetNetwork(
            {"image": placeholder_input},
            conv_width=0.75,
            conv_width2=1.00,
            trainable=trainable,
        )
        pretrain_path = (
            "pretrained/mobilenet_v1_0.75_224_2017_06_14/mobilenet_v1_0.75_224.ckpt"
        )
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_fast":
        net = MobilenetNetwork(
            {"image": placeholder_input},
            conv_width=0.5,
            conv_width2=0.5,
            trainable=trainable,
        )
        pretrain_path = (
            "pretrained/mobilenet_v1_0.75_224_2017_06_14/mobilenet_v1_0.75_224.ckpt"
        )
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_accurate":
        net = MobilenetNetwork(
            {"image": placeholder_input},
            conv_width=1.00,
            conv_width2=1.00,
            trainable=trainable,
        )
        pretrain_path = (
            "pretrained/mobilenet_v1_1.0_224_2017_06_14/mobilenet_v1_1.0_224.ckpt"
        )
        last_layer = "MConv_Stage6_L{aux}_5"

    elif type == "mobilenet_thin":
        net = MobilenetNetworkThin(
            {"image": placeholder_input},
            conv_width=0.75,
            conv_width2=0.50,
            trainable=trainable,
        )
        pretrain_path = (
            "pretrained/mobilenet_v1_0.75_224_2017_06_14/mobilenet_v1_0.75_224.ckpt"
        )
        last_layer = "MConv_Stage6_L{aux}_5"

    elif type in [
        "mobilenet_v2_w1.4_r1.0",
        "mobilenet_v2_large",
        "mobilenet_v2_large_quantize",
    ]:  # m_v2_large
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=1.4,
            conv_width2=1.0,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_1.4_224/mobilenet_v2_1.4_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w1.4_r0.5":
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=1.4,
            conv_width2=0.5,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_1.4_224/mobilenet_v2_1.4_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w1.0_r1.0":
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=1.0,
            conv_width2=1.0,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_1.0_224/mobilenet_v2_1.0_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w1.0_r0.75":
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=1.0,
            conv_width2=0.75,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_1.0_224/mobilenet_v2_1.0_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w1.0_r0.5":
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=1.0,
            conv_width2=0.5,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_1.0_224/mobilenet_v2_1.0_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w0.75_r0.75":
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=0.75,
            conv_width2=0.75,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_0.75_224/mobilenet_v2_0.75_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_w0.5_r0.5" or type == "mobilenet_v2_small":  # m_v2_fast
        net = Mobilenetv2Network(
            {"image": placeholder_input},
            conv_width=0.5,
            conv_width2=0.5,
            trainable=trainable,
        )
        pretrain_path = "pretrained/mobilenet_v2_0.5_224/mobilenet_v2_0.5_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"

    elif type == "mobilenet_v2_1.4":
        net = Mobilenetv2Network(
            {"image": placeholder_input}, conv_width=1.4, trainable=trainable
        )
        pretrain_path = "pretrained/mobilenet_v2_1.4_224/mobilenet_v2_1.4_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_1.0":
        net = Mobilenetv2Network(
            {"image": placeholder_input}, conv_width=1.0, trainable=trainable
        )
        pretrain_path = "pretrained/mobilenet_v2_1.0_224/mobilenet_v2_1.0_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_0.75":
        net = Mobilenetv2Network(
            {"image": placeholder_input}, conv_width=0.75, trainable=trainable
        )
        pretrain_path = "pretrained/mobilenet_v2_0.75_224/mobilenet_v2_0.75_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"
    elif type == "mobilenet_v2_0.5":
        net = Mobilenetv2Network(
            {"image": placeholder_input}, conv_width=0.5, trainable=trainable
        )
        pretrain_path = "pretrained/mobilenet_v2_0.5_224/mobilenet_v2_0.5_224.ckpt"
        last_layer = "MConv_Stage6_L{aux}_5"

    elif type in ["cmu", "openpose"]:
        net = CmuNetwork({"image": placeholder_input}, trainable=trainable)
        pretrain_path = "numpy/openpose_coco.npy"
        last_layer = "Mconv7_stage6_L{aux}"
    elif type in ["cmu_quantize", "openpose_quantize"]:
        net = CmuNetwork({"image": placeholder_input}, trainable=trainable)
        pretrain_path = "train/cmu/bs8_lr0.0001_q_e80/model_latest-18000"
        last_layer = "Mconv7_stage6_L{aux}"
    elif type == "vgg":
        net = CmuNetwork({"image": placeholder_input}, trainable=trainable)
        pretrain_path = "numpy/openpose_vgg16.npy"
        last_layer = "Mconv7_stage6_L{aux}"

    else:
        raise Exception("Invalid Model Name.")

    pretrain_path_full = os.path.join(_get_base_path(), pretrain_path)
    if sess_for_load is not None:
        if type in ["cmu", "vgg", "openpose"]:
            if not os.path.isfile(pretrain_path_full):
                raise Exception(
                    "Model file doesn't exist, path=%s" % pretrain_path_full
                )
            net.load(os.path.join(_get_base_path(), pretrain_path), sess_for_load)
        else:
            try:
                s = "%dx%d" % (placeholder_input.shape[2], placeholder_input.shape[1])
            except:
                s = ""
            ckpts = {
                "mobilenet": "trained/mobilenet_%s/model-246038" % s,
                "mobilenet_thin": "trained/mobilenet_thin_%s/model-449003" % s,
                "mobilenet_fast": "trained/mobilenet_fast_%s/model-189000" % s,
                "mobilenet_accurate": "trained/mobilenet_accurate/model-170000",
                "mobilenet_v2_w1.4_r0.5": "trained/mobilenet_v2_w1.4_r0.5/model_latest-380401",
                "mobilenet_v2_large": "trained/mobilenet_v2_w1.4_r1.0/model-570000",
                "mobilenet_v2_small": "trained/mobilenet_v2_w0.5_r0.5/model_latest-380401",
            }
            ckpt_path = os.path.join(_get_base_path(), ckpts[type])
            loader = tf.train.Saver()
            try:
                loader.restore(sess_for_load, ckpt_path)
            except Exception as e:
                raise Exception(
                    "Fail to load model files. \npath=%s\nerr=%s" % (ckpt_path, str(e))
                )

    return net, pretrain_path_full, last_layer


def get_graph_path(model_name):
    dyn_graph_path = {
        "cmu": "graph/cmu/graph_opt.pb",
        "openpose_quantize": "graph/cmu/graph_opt_q.pb",
        "mobilenet_thin": "graph/mobilenet_thin/graph_opt.pb",
        "mobilenet_v2_large": "graph/mobilenet_v2_large/graph_opt.pb",
        "mobilenet_v2_large_r0.5": "graph/mobilenet_v2_large/graph_r0.5_opt.pb",
        "mobilenet_v2_large_quantize": "graph/mobilenet_v2_large/graph_opt_q.pb",
        "mobilenet_v2_small": "graph/mobilenet_v2_small/graph_opt.pb",
    }

    base_data_dir = dirname(dirname(abspath(__file__)))
    if os.path.exists(os.path.join(base_data_dir, "models")):
        base_data_dir = os.path.join(base_data_dir, "models")
    else:
        base_data_dir = os.path.join(base_data_dir, "tf_pose_data")

    graph_path = os.path.join(base_data_dir, dyn_graph_path[model_name])
    if os.path.isfile(graph_path):
        return graph_path

    raise Exception("Graph file doesn't exist, path=%s" % graph_path)


def model_wh(resolution_str):
    width, height = map(int, resolution_str.split("x"))
    if width % 16 != 0 or height % 16 != 0:
        raise Exception(
            "Width and height should be multiples of 16. w=%d, h=%d" % (width, height)
        )
    return int(width), int(height)



import sys
import os
import time
from collections import OrderedDict

import numpy as np
import logging
import argparse
import json, re
from tqdm import tqdm

from tf_pose.common import read_imgfile
from tf_pose.estimator import TfPoseEstimator
from tf_pose.networks import model_wh, get_graph_path

from pycocotools.coco import COCO
from pycocotools.cocoeval import COCOeval

logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s %(message)s")

logger = logging.getLogger("TfPoseEstimator-Video")
logger.setLevel(logging.INFO)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)

eval_size = -1


def round_int(val):
    return int(round(val))


def write_coco_json(human, image_w, image_h):
    keypoints = []
    coco_ids = [0, 15, 14, 17, 16, 5, 2, 6, 3, 7, 4, 11, 8, 12, 9, 13, 10]
    for coco_id in coco_ids:
        if coco_id not in human.body_parts.keys():
            keypoints.extend([0, 0, 0])
            continue
        body_part = human.body_parts[coco_id]
        keypoints.extend(
            [round_int(body_part.x * image_w), round_int(body_part.y * image_h), 2]
        )
    return keypoints


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Tensorflow Openpose Inference")
    parser.add_argument(
        "--resize",
        type=str,
        default="0x0",
        help="if provided, resize images before they are processed. default=0x0, Recommends : 432x368 or 656x368 or 1312x736 ",
    )
    parser.add_argument(
        "--resize-out-ratio",
        type=float,
        default=8.0,
        help="if provided, resize heatmaps before they are post-processed. default=8.0",
    )
    parser.add_argument(
        "--model",
        type=str,
        default="cmu",
        help="cmu / mobilenet_thin / mobilenet_v2_large",
    )
    parser.add_argument("--cocoyear", type=str, default="2014")
    parser.add_argument("--coco-dir", type=str, default="/data/public/rw/coco/")
    parser.add_argument("--data-idx", type=int, default=-1)
    parser.add_argument("--multi-scale", type=bool, default=False)
    args = parser.parse_args()

    cocoyear_list = ["2014", "2017"]
    if args.cocoyear not in cocoyear_list:
        logger.error("cocoyear should be one of %s" % str(cocoyear_list))
        sys.exit(-1)

    # TODO : Scales

    image_dir = args.coco_dir + "val%s/" % args.cocoyear
    coco_json_file = (
        args.coco_dir + "annotations/person_keypoints_val%s.json" % args.cocoyear
    )
    cocoGt = COCO(coco_json_file)
    catIds = cocoGt.getCatIds(catNms=["person"])
    keys = cocoGt.getImgIds(catIds=catIds)
    if args.data_idx < 0:
        if eval_size > 0:
            keys = keys[:eval_size]  # only use the first #eval_size elements.
        pass
    else:
        keys = [keys[args.data_idx]]
    logger.info("validation %s set size=%d" % (coco_json_file, len(keys)))
    write_json = "../etcs/%s_%s_%0.1f.json" % (
        args.model,
        args.resize,
        args.resize_out_ratio,
    )

    logger.debug("initialization %s : %s" % (args.model, get_graph_path(args.model)))
    w, h = model_wh(args.resize)
    if w == 0 or h == 0:
        e = TfPoseEstimator(get_graph_path(args.model), target_size=(432, 368))
    else:
        e = TfPoseEstimator(get_graph_path(args.model), target_size=(w, h))

    print("FLOPs: ", e.get_flops())

    result = []
    tqdm_keys = tqdm(keys)
    for i, k in enumerate(tqdm_keys):
        img_meta = cocoGt.loadImgs(k)[0]
        img_idx = img_meta["id"]

        img_name = os.path.join(image_dir, img_meta["file_name"])
        image = read_imgfile(img_name, None, None)
        if image is None:
            logger.error("image not found, path=%s" % img_name)
            sys.exit(-1)

        # inference the image with the specified network
        t = time.time()
        humans = e.inference(
            image,
            resize_to_default=(w > 0 and h > 0),
            upsample_size=args.resize_out_ratio,
        )
        elapsed = time.time() - t

        scores = 0
        ann_idx = cocoGt.getAnnIds(imgIds=[img_idx], catIds=[1])
        anns = cocoGt.loadAnns(ann_idx)
        for human in humans:
            item = {
                "image_id": img_idx,
                "category_id": 1,
                "keypoints": write_coco_json(
                    human, img_meta["width"], img_meta["height"]
                ),
                "score": human.score,
            }
            result.append(item)
            scores += item["score"]

        avg_score = scores / len(humans) if len(humans) > 0 else 0
        tqdm_keys.set_postfix(
            OrderedDict({"inference time": elapsed, "score": avg_score})
        )
        if args.data_idx >= 0:
            logger.info("score:", k, len(humans), len(anns), avg_score)

            import matplotlib.pyplot as plt

            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            plt.imshow(e.draw_humans(image, humans, True))

            a = fig.add_subplot(2, 3, 2)
            # plt.imshow(cv2.resize(image, (e.heatMat.shape[1], e.heatMat.shape[0])), alpha=0.5)
            tmp = np.amax(e.heatMat[:, :, :-1], axis=2)
            plt.imshow(tmp, cmap=plt.cm.gray, alpha=0.5)
            plt.colorbar()

            tmp2 = e.pafMat.transpose((2, 0, 1))
            tmp2_odd = np.amax(np.absolute(tmp2[::2, :, :]), axis=0)
            tmp2_even = np.amax(np.absolute(tmp2[1::2, :, :]), axis=0)

            a = fig.add_subplot(2, 3, 4)
            a.set_title("Vectormap-x")
            # plt.imshow(CocoPose.get_bgimg(inp, target_size=(vectmap.shape[1], vectmap.shape[0])), alpha=0.5)
            plt.imshow(tmp2_odd, cmap=plt.cm.gray, alpha=0.5)
            plt.colorbar()

            a = fig.add_subplot(2, 3, 5)
            a.set_title("Vectormap-y")
            # plt.imshow(CocoPose.get_bgimg(inp, target_size=(vectmap.shape[1], vectmap.shape[0])), alpha=0.5)
            plt.imshow(tmp2_even, cmap=plt.cm.gray, alpha=0.5)
            plt.colorbar()

            plt.show()

    fp = open(write_json, "w")
    json.dump(result, fp)
    fp.close()

    cocoDt = cocoGt.loadRes(write_json)
    cocoEval = COCOeval(cocoGt, cocoDt, "keypoints")
    cocoEval.params.imgIds = keys
    cocoEval.evaluate()
    cocoEval.accumulate()
    cocoEval.summarize()

    print("".join(["%11.4f |" % x for x in cocoEval.stats]))

    pred = json.load(open(write_json, "r"))



from distutils.core import setup, Extension
import numpy
import os

# os.environ['CC'] = 'g++';
setup(
    name="pafprocess_ext",
    version="1.0",
    ext_modules=[
        Extension(
            "_pafprocess",
            ["pafprocess.cpp", "pafprocess.i"],
            swig_opts=["-c++"],
            depends=["pafprocess.h"],
            include_dirs=[numpy.get_include(), "."],
        )
    ],
    py_modules=["pafprocess"],
)



from .SlidingWindow import DimOrder, SlidingWindow, generate, generateForSize
from .WindowDistance import generateDistanceMatrix
from .RectangleUtils import *
from .ArrayUtils import *
from .Batching import *
from .Merging import *



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info

if _swig_python_version_info >= (2, 7, 0):

    def swig_import_helper():
        import importlib

        pkg = __name__.rpartition(".")[0]
        mname = ".".join((pkg, "_pafprocess")).lstrip(".")
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module("_pafprocess")

    _pafprocess = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):

    def swig_import_helper():
        from os.path import dirname
        import imp

        fp = None
        try:
            fp, pathname, description = imp.find_module(
                "_pafprocess", [dirname(__file__)]
            )
        except ImportError:
            import _pafprocess

            return _pafprocess
        try:
            _mod = imp.load_module("_pafprocess", fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod

    _pafprocess = swig_import_helper()
    del swig_import_helper
else:
    import _pafprocess
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if name == "thisown":
        return self.this.own(value)
    if name == "this":
        if type(value).__name__ == "SwigPyObject":
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if not static:
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if name == "thisown":
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError(
        "'%s' object has no attribute '%s'" % (class_type.__name__, name)
    )


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (
        self.__class__.__module__,
        self.__class__.__name__,
        strthis,
    )


try:
    _object = object
    _newclass = 1
except __builtin__.Exception:

    class _object:
        pass

    _newclass = 0


class Peak(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Peak, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Peak, name)
    __repr__ = _swig_repr
    __swig_setmethods__["x"] = _pafprocess.Peak_x_set
    __swig_getmethods__["x"] = _pafprocess.Peak_x_get
    if _newclass:
        x = _swig_property(_pafprocess.Peak_x_get, _pafprocess.Peak_x_set)
    __swig_setmethods__["y"] = _pafprocess.Peak_y_set
    __swig_getmethods__["y"] = _pafprocess.Peak_y_get
    if _newclass:
        y = _swig_property(_pafprocess.Peak_y_get, _pafprocess.Peak_y_set)
    __swig_setmethods__["score"] = _pafprocess.Peak_score_set
    __swig_getmethods__["score"] = _pafprocess.Peak_score_get
    if _newclass:
        score = _swig_property(_pafprocess.Peak_score_get, _pafprocess.Peak_score_set)
    __swig_setmethods__["id"] = _pafprocess.Peak_id_set
    __swig_getmethods__["id"] = _pafprocess.Peak_id_get
    if _newclass:
        id = _swig_property(_pafprocess.Peak_id_get, _pafprocess.Peak_id_set)

    def __init__(self):
        this = _pafprocess.new_Peak()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    __swig_destroy__ = _pafprocess.delete_Peak
    __del__ = lambda self: None


Peak_swigregister = _pafprocess.Peak_swigregister
Peak_swigregister(Peak)
cvar = _pafprocess.cvar
THRESH_HEAT = cvar.THRESH_HEAT
THRESH_VECTOR_SCORE = cvar.THRESH_VECTOR_SCORE
THRESH_VECTOR_CNT1 = cvar.THRESH_VECTOR_CNT1
THRESH_PART_CNT = cvar.THRESH_PART_CNT
THRESH_HUMAN_SCORE = cvar.THRESH_HUMAN_SCORE
NUM_PART = cvar.NUM_PART
STEP_PAF = cvar.STEP_PAF
COCOPAIRS_SIZE = cvar.COCOPAIRS_SIZE
COCOPAIRS_NET = cvar.COCOPAIRS_NET
COCOPAIRS = cvar.COCOPAIRS


class VectorXY(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, VectorXY, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, VectorXY, name)
    __repr__ = _swig_repr
    __swig_setmethods__["x"] = _pafprocess.VectorXY_x_set
    __swig_getmethods__["x"] = _pafprocess.VectorXY_x_get
    if _newclass:
        x = _swig_property(_pafprocess.VectorXY_x_get, _pafprocess.VectorXY_x_set)
    __swig_setmethods__["y"] = _pafprocess.VectorXY_y_set
    __swig_getmethods__["y"] = _pafprocess.VectorXY_y_get
    if _newclass:
        y = _swig_property(_pafprocess.VectorXY_y_get, _pafprocess.VectorXY_y_set)

    def __init__(self):
        this = _pafprocess.new_VectorXY()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    __swig_destroy__ = _pafprocess.delete_VectorXY
    __del__ = lambda self: None


VectorXY_swigregister = _pafprocess.VectorXY_swigregister
VectorXY_swigregister(VectorXY)


class ConnectionCandidate(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(
        self, ConnectionCandidate, name, value
    )
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ConnectionCandidate, name)
    __repr__ = _swig_repr
    __swig_setmethods__["idx1"] = _pafprocess.ConnectionCandidate_idx1_set
    __swig_getmethods__["idx1"] = _pafprocess.ConnectionCandidate_idx1_get
    if _newclass:
        idx1 = _swig_property(
            _pafprocess.ConnectionCandidate_idx1_get,
            _pafprocess.ConnectionCandidate_idx1_set,
        )
    __swig_setmethods__["idx2"] = _pafprocess.ConnectionCandidate_idx2_set
    __swig_getmethods__["idx2"] = _pafprocess.ConnectionCandidate_idx2_get
    if _newclass:
        idx2 = _swig_property(
            _pafprocess.ConnectionCandidate_idx2_get,
            _pafprocess.ConnectionCandidate_idx2_set,
        )
    __swig_setmethods__["score"] = _pafprocess.ConnectionCandidate_score_set
    __swig_getmethods__["score"] = _pafprocess.ConnectionCandidate_score_get
    if _newclass:
        score = _swig_property(
            _pafprocess.ConnectionCandidate_score_get,
            _pafprocess.ConnectionCandidate_score_set,
        )
    __swig_setmethods__["etc"] = _pafprocess.ConnectionCandidate_etc_set
    __swig_getmethods__["etc"] = _pafprocess.ConnectionCandidate_etc_get
    if _newclass:
        etc = _swig_property(
            _pafprocess.ConnectionCandidate_etc_get,
            _pafprocess.ConnectionCandidate_etc_set,
        )

    def __init__(self):
        this = _pafprocess.new_ConnectionCandidate()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    __swig_destroy__ = _pafprocess.delete_ConnectionCandidate
    __del__ = lambda self: None


ConnectionCandidate_swigregister = _pafprocess.ConnectionCandidate_swigregister
ConnectionCandidate_swigregister(ConnectionCandidate)


class Connection(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Connection, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Connection, name)
    __repr__ = _swig_repr
    __swig_setmethods__["cid1"] = _pafprocess.Connection_cid1_set
    __swig_getmethods__["cid1"] = _pafprocess.Connection_cid1_get
    if _newclass:
        cid1 = _swig_property(
            _pafprocess.Connection_cid1_get, _pafprocess.Connection_cid1_set
        )
    __swig_setmethods__["cid2"] = _pafprocess.Connection_cid2_set
    __swig_getmethods__["cid2"] = _pafprocess.Connection_cid2_get
    if _newclass:
        cid2 = _swig_property(
            _pafprocess.Connection_cid2_get, _pafprocess.Connection_cid2_set
        )
    __swig_setmethods__["score"] = _pafprocess.Connection_score_set
    __swig_getmethods__["score"] = _pafprocess.Connection_score_get
    if _newclass:
        score = _swig_property(
            _pafprocess.Connection_score_get, _pafprocess.Connection_score_set
        )
    __swig_setmethods__["peak_id1"] = _pafprocess.Connection_peak_id1_set
    __swig_getmethods__["peak_id1"] = _pafprocess.Connection_peak_id1_get
    if _newclass:
        peak_id1 = _swig_property(
            _pafprocess.Connection_peak_id1_get, _pafprocess.Connection_peak_id1_set
        )
    __swig_setmethods__["peak_id2"] = _pafprocess.Connection_peak_id2_set
    __swig_getmethods__["peak_id2"] = _pafprocess.Connection_peak_id2_get
    if _newclass:
        peak_id2 = _swig_property(
            _pafprocess.Connection_peak_id2_get, _pafprocess.Connection_peak_id2_set
        )

    def __init__(self):
        this = _pafprocess.new_Connection()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    __swig_destroy__ = _pafprocess.delete_Connection
    __del__ = lambda self: None


Connection_swigregister = _pafprocess.Connection_swigregister
Connection_swigregister(Connection)


def process_paf(p1, h1, f1):
    return _pafprocess.process_paf(p1, h1, f1)


process_paf = _pafprocess.process_paf


def get_num_humans():
    return _pafprocess.get_num_humans()


get_num_humans = _pafprocess.get_num_humans


def get_part_cid(human_id, part_id):
    return _pafprocess.get_part_cid(human_id, part_id)


get_part_cid = _pafprocess.get_part_cid


def get_score(human_id):
    return _pafprocess.get_score(human_id)


get_score = _pafprocess.get_score


def get_part_x(cid):
    return _pafprocess.get_part_x(cid)


get_part_x = _pafprocess.get_part_x


def get_part_y(cid):
    return _pafprocess.get_part_y(cid)


get_part_y = _pafprocess.get_part_y


def get_part_score(cid):
    return _pafprocess.get_part_score(cid)


get_part_score = _pafprocess.get_part_score
# This file is compatible with both classic and new-style classes.



import logging
import math
import multiprocessing
import struct
import sys
import threading

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from contextlib import contextmanager

import os
import random
import requests
import cv2
import numpy as np
import time

import tensorflow as tf

from tensorpack.dataflow import MultiThreadMapData
from tensorpack.dataflow.image import MapDataComponent
from tensorpack.dataflow.common import BatchData, MapData
from tensorpack.dataflow.parallel import PrefetchData
from tensorpack.dataflow.base import RNGDataFlow, DataFlowTerminated

from pycocotools.coco import COCO
from pose_augment import (
    pose_flip,
    pose_rotation,
    pose_to_img,
    pose_crop_random,
    pose_resize_shortestedge_random,
    pose_resize_shortestedge_fixed,
    pose_crop_center,
    pose_random_scale,
)
from numba import jit

logging.getLogger("requests").setLevel(logging.WARNING)
logger = logging.getLogger("pose_dataset")
logger.setLevel(logging.INFO)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)

mplset = False


class CocoMetadata:
    # __coco_parts = 57
    __coco_parts = 19
    __coco_vecs = list(
        zip(
            [2, 9, 10, 2, 12, 13, 2, 3, 4, 3, 2, 6, 7, 6, 2, 1, 1, 15, 16],
            [9, 10, 11, 12, 13, 14, 3, 4, 5, 17, 6, 7, 8, 18, 1, 15, 16, 17, 18],
        )
    )

    @staticmethod
    def parse_float(four_np):
        assert len(four_np) == 4
        return struct.unpack("<f", bytes(four_np))[0]

    @staticmethod
    def parse_floats(four_nps, adjust=0):
        assert len(four_nps) % 4 == 0
        return [
            (CocoMetadata.parse_float(four_nps[x * 4 : x * 4 + 4]) + adjust)
            for x in range(len(four_nps) // 4)
        ]

    def __init__(self, idx, img_url, img_meta, annotations, sigma):
        self.idx = idx
        self.img_url = img_url
        self.img = None
        self.sigma = sigma

        self.height = int(img_meta["height"])
        self.width = int(img_meta["width"])

        joint_list = []
        for ann in annotations:
            if ann.get("num_keypoints", 0) == 0:
                continue

            kp = np.array(ann["keypoints"])
            xs = kp[0::3]
            ys = kp[1::3]
            vs = kp[2::3]

            joint_list.append(
                [(x, y) if v >= 1 else (-1000, -1000) for x, y, v in zip(xs, ys, vs)]
            )

        self.joint_list = []
        transform = list(
            zip(
                [1, 6, 7, 9, 11, 6, 8, 10, 13, 15, 17, 12, 14, 16, 3, 2, 5, 4],
                [1, 7, 7, 9, 11, 6, 8, 10, 13, 15, 17, 12, 14, 16, 3, 2, 5, 4],
            )
        )
        for prev_joint in joint_list:
            new_joint = []
            for idx1, idx2 in transform:
                j1 = prev_joint[idx1 - 1]
                j2 = prev_joint[idx2 - 1]

                if j1[0] <= 0 or j1[1] <= 0 or j2[0] <= 0 or j2[1] <= 0:
                    new_joint.append((-1000, -1000))
                else:
                    new_joint.append(((j1[0] + j2[0]) / 2, (j1[1] + j2[1]) / 2))

            new_joint.append((-1000, -1000))
            self.joint_list.append(new_joint)

        # logger.debug('joint size=%d' % len(self.joint_list))

    @jit
    def get_heatmap(self, target_size):
        heatmap = np.zeros(
            (CocoMetadata.__coco_parts, self.height, self.width), dtype=np.float32
        )

        for joints in self.joint_list:
            for idx, point in enumerate(joints):
                if point[0] < 0 or point[1] < 0:
                    continue
                CocoMetadata.put_heatmap(heatmap, idx, point, self.sigma)

        heatmap = heatmap.transpose((1, 2, 0))

        # background
        heatmap[:, :, -1] = np.clip(1 - np.amax(heatmap, axis=2), 0.0, 1.0)

        if target_size:
            heatmap = cv2.resize(heatmap, target_size, interpolation=cv2.INTER_AREA)

        return heatmap.astype(np.float16)

    @staticmethod
    @jit(nopython=True)
    def put_heatmap(heatmap, plane_idx, center, sigma):
        center_x, center_y = center
        _, height, width = heatmap.shape[:3]

        th = 4.6052
        delta = math.sqrt(th * 2)

        x0 = int(max(0, center_x - delta * sigma))
        y0 = int(max(0, center_y - delta * sigma))

        x1 = int(min(width, center_x + delta * sigma))
        y1 = int(min(height, center_y + delta * sigma))

        for y in range(y0, y1):
            for x in range(x0, x1):
                d = (x - center_x) ** 2 + (y - center_y) ** 2
                exp = d / 2.0 / sigma / sigma
                if exp > th:
                    continue
                heatmap[plane_idx][y][x] = max(heatmap[plane_idx][y][x], math.exp(-exp))
                heatmap[plane_idx][y][x] = min(heatmap[plane_idx][y][x], 1.0)

    @jit
    def get_vectormap(self, target_size):
        vectormap = np.zeros(
            (CocoMetadata.__coco_parts * 2, self.height, self.width), dtype=np.float32
        )
        countmap = np.zeros(
            (CocoMetadata.__coco_parts, self.height, self.width), dtype=np.int16
        )
        for joints in self.joint_list:
            for plane_idx, (j_idx1, j_idx2) in enumerate(CocoMetadata.__coco_vecs):
                j_idx1 -= 1
                j_idx2 -= 1

                center_from = joints[j_idx1]
                center_to = joints[j_idx2]

                if (
                    center_from[0] < -100
                    or center_from[1] < -100
                    or center_to[0] < -100
                    or center_to[1] < -100
                ):
                    continue

                CocoMetadata.put_vectormap(
                    vectormap, countmap, plane_idx, center_from, center_to
                )

        vectormap = vectormap.transpose((1, 2, 0))
        nonzeros = np.nonzero(countmap)
        for p, y, x in zip(nonzeros[0], nonzeros[1], nonzeros[2]):
            if countmap[p][y][x] <= 0:
                continue
            vectormap[y][x][p * 2 + 0] /= countmap[p][y][x]
            vectormap[y][x][p * 2 + 1] /= countmap[p][y][x]

        if target_size:
            vectormap = cv2.resize(vectormap, target_size, interpolation=cv2.INTER_AREA)

        return vectormap.astype(np.float16)

    @staticmethod
    @jit(nopython=True)
    def put_vectormap(
        vectormap, countmap, plane_idx, center_from, center_to, threshold=8
    ):
        _, height, width = vectormap.shape[:3]

        vec_x = center_to[0] - center_from[0]
        vec_y = center_to[1] - center_from[1]

        min_x = max(0, int(min(center_from[0], center_to[0]) - threshold))
        min_y = max(0, int(min(center_from[1], center_to[1]) - threshold))

        max_x = min(width, int(max(center_from[0], center_to[0]) + threshold))
        max_y = min(height, int(max(center_from[1], center_to[1]) + threshold))

        norm = math.sqrt(vec_x ** 2 + vec_y ** 2)
        if norm == 0:
            return

        vec_x /= norm
        vec_y /= norm

        for y in range(min_y, max_y):
            for x in range(min_x, max_x):
                bec_x = x - center_from[0]
                bec_y = y - center_from[1]
                dist = abs(bec_x * vec_y - bec_y * vec_x)

                if dist > threshold:
                    continue

                countmap[plane_idx][y][x] += 1

                vectormap[plane_idx * 2 + 0][y][x] = vec_x
                vectormap[plane_idx * 2 + 1][y][x] = vec_y


class CocoPose(RNGDataFlow):
    @staticmethod
    def display_image(inp, heatmap, vectmap, as_numpy=False):
        global mplset
        # if as_numpy and not mplset:
        #     import matplotlib as mpl
        #     mpl.use('Agg')
        mplset = True
        import matplotlib.pyplot as plt

        fig = plt.figure()
        a = fig.add_subplot(2, 2, 1)
        a.set_title("Image")
        plt.imshow(CocoPose.get_bgimg(inp))

        a = fig.add_subplot(2, 2, 2)
        a.set_title("Heatmap")
        plt.imshow(
            CocoPose.get_bgimg(inp, target_size=(heatmap.shape[1], heatmap.shape[0])),
            alpha=0.5,
        )
        tmp = np.amax(heatmap, axis=2)
        plt.imshow(tmp, cmap=plt.cm.gray, alpha=0.5)
        plt.colorbar()

        tmp2 = vectmap.transpose((2, 0, 1))
        tmp2_odd = np.amax(np.absolute(tmp2[::2, :, :]), axis=0)
        tmp2_even = np.amax(np.absolute(tmp2[1::2, :, :]), axis=0)

        a = fig.add_subplot(2, 2, 3)
        a.set_title("Vectormap-x")
        plt.imshow(
            CocoPose.get_bgimg(inp, target_size=(vectmap.shape[1], vectmap.shape[0])),
            alpha=0.5,
        )
        plt.imshow(tmp2_odd, cmap=plt.cm.gray, alpha=0.5)
        plt.colorbar()

        a = fig.add_subplot(2, 2, 4)
        a.set_title("Vectormap-y")
        plt.imshow(
            CocoPose.get_bgimg(inp, target_size=(vectmap.shape[1], vectmap.shape[0])),
            alpha=0.5,
        )
        plt.imshow(tmp2_even, cmap=plt.cm.gray, alpha=0.5)
        plt.colorbar()

        if not as_numpy:
            plt.show()
        else:
            fig.canvas.draw()
            data = np.fromstring(fig.canvas.tostring_rgb(), dtype=np.uint8, sep="")
            data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))
            fig.clear()
            plt.close()
            return data

    @staticmethod
    def get_bgimg(inp, target_size=None):
        inp = cv2.cvtColor(inp.astype(np.uint8), cv2.COLOR_BGR2RGB)
        if target_size:
            inp = cv2.resize(inp, target_size, interpolation=cv2.INTER_AREA)
        return inp

    def __init__(
        self, path, img_path=None, is_train=True, decode_img=True, only_idx=-1
    ):
        self.is_train = is_train
        self.decode_img = decode_img
        self.only_idx = only_idx

        if is_train:
            whole_path = os.path.join(path, "person_keypoints_train2017.json")
        else:
            whole_path = os.path.join(path, "person_keypoints_val2017.json")
        self.img_path = (img_path if img_path is not None else "") + (
            "train2017/" if is_train else "val2017/"
        )
        self.coco = COCO(whole_path)

        logger.info("%s dataset %d" % (path, self.size()))

    def size(self):
        return len(self.coco.imgs)

    def get_data(self):
        idxs = np.arange(self.size())
        if self.is_train:
            self.rng.shuffle(idxs)
        else:
            pass

        keys = list(self.coco.imgs.keys())
        for idx in idxs:
            img_meta = self.coco.imgs[keys[idx]]
            img_idx = img_meta["id"]
            ann_idx = self.coco.getAnnIds(imgIds=img_idx)

            if "http://" in self.img_path:
                img_url = self.img_path + img_meta["file_name"]
            else:
                img_url = os.path.join(self.img_path, img_meta["file_name"])

            anns = self.coco.loadAnns(ann_idx)
            meta = CocoMetadata(idx, img_url, img_meta, anns, sigma=8.0)

            total_keypoints = sum([ann.get("num_keypoints", 0) for ann in anns])
            if total_keypoints == 0 and random.uniform(0, 1) > 0.2:
                continue

            yield [meta]


class MPIIPose(RNGDataFlow):
    def __init__(self):
        pass

    def size(self):
        pass

    def get_data(self):
        pass


def read_image_url(metas):
    for meta in metas:
        img_str = None
        if "http://" in meta.img_url:
            # print(meta.img_url)
            for _ in range(10):
                try:
                    resp = requests.get(meta.img_url)
                    if resp.status_code // 100 != 2:
                        logger.warning(
                            "request failed code=%d url=%s"
                            % (resp.status_code, meta.img_url)
                        )
                        time.sleep(1.0)
                        continue
                    img_str = resp.content
                    break
                except Exception as e:
                    logger.warning(
                        "request failed url=%s, err=%s" % (meta.img_url, str(e))
                    )
        else:
            img_str = open(meta.img_url, "rb").read()

        if not img_str:
            logger.warning("image not read, path=%s" % meta.img_url)
            raise Exception()

        nparr = np.fromstring(img_str, np.uint8)
        meta.img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
    return metas


def get_dataflow(path, is_train, img_path=None):
    ds = CocoPose(path, img_path, is_train)  # read data from lmdb
    if is_train:
        ds = MapData(ds, read_image_url)
        ds = MapDataComponent(ds, pose_random_scale)
        ds = MapDataComponent(ds, pose_rotation)
        ds = MapDataComponent(ds, pose_flip)
        ds = MapDataComponent(ds, pose_resize_shortestedge_random)
        ds = MapDataComponent(ds, pose_crop_random)
        ds = MapData(ds, pose_to_img)
        # augs = [
        #     imgaug.RandomApplyAug(imgaug.RandomChooseAug([
        #         imgaug.GaussianBlur(max_size=3)
        #     ]), 0.7)
        # ]
        # ds = AugmentImageComponent(ds, augs)
        ds = PrefetchData(ds, 1000, multiprocessing.cpu_count() * 1)
    else:
        ds = MultiThreadMapData(
            ds, nr_thread=16, map_func=read_image_url, buffer_size=1000
        )
        ds = MapDataComponent(ds, pose_resize_shortestedge_fixed)
        ds = MapDataComponent(ds, pose_crop_center)
        ds = MapData(ds, pose_to_img)
        ds = PrefetchData(ds, 100, multiprocessing.cpu_count() // 4)

    return ds


def _get_dataflow_onlyread(path, is_train, img_path=None):
    ds = CocoPose(path, img_path, is_train)  # read data from lmdb
    ds = MapData(ds, read_image_url)
    ds = MapData(ds, pose_to_img)
    # ds = PrefetchData(ds, 1000, multiprocessing.cpu_count() * 4)
    return ds


def get_dataflow_batch(path, is_train, batchsize, img_path=None):
    logger.info("dataflow img_path=%s" % img_path)
    ds = get_dataflow(path, is_train, img_path=img_path)
    ds = BatchData(ds, batchsize)
    # if is_train:
    #     ds = PrefetchData(ds, 10, 2)
    # else:
    #     ds = PrefetchData(ds, 50, 2)

    return ds


class DataFlowToQueue(threading.Thread):
    def __init__(self, ds, placeholders, queue_size=5):
        super().__init__()
        self.daemon = True

        self.ds = ds
        self.placeholders = placeholders
        self.queue = tf.FIFOQueue(
            queue_size,
            [ph.dtype for ph in placeholders],
            shapes=[ph.get_shape() for ph in placeholders],
        )
        self.op = self.queue.enqueue(placeholders)
        self.close_op = self.queue.close(cancel_pending_enqueues=True)

        self._coord = None
        self._sess = None

        self.last_dp = None

    @contextmanager
    def default_sess(self):
        if self._sess:
            with self._sess.as_default():
                yield
        else:
            logger.warning(
                "DataFlowToQueue {} wasn't under a default session!".format(self.name)
            )
            yield

    def size(self):
        return self.queue.size()

    def start(self):
        self._sess = tf.get_default_session()
        super().start()

    def set_coordinator(self, coord):
        self._coord = coord

    def run(self):
        with self.default_sess():
            try:
                while not self._coord.should_stop():
                    try:
                        self.ds.reset_state()
                        while True:
                            for dp in self.ds.get_data():
                                feed = dict(zip(self.placeholders, dp))
                                self.op.run(feed_dict=feed)
                                self.last_dp = dp
                    except (
                        tf.errors.CancelledError,
                        tf.errors.OutOfRangeError,
                        DataFlowTerminated,
                    ):
                        logger.error(
                            "err type1, placeholders={}".format(self.placeholders)
                        )
                        sys.exit(-1)
                    except Exception as e:
                        logger.error(
                            "err type2, err={}, placeholders={}".format(
                                str(e), self.placeholders
                            )
                        )
                        if isinstance(e, RuntimeError) and "closed Session" in str(e):
                            pass
                        else:
                            logger.exception(
                                "Exception in {}:{}".format(self.name, str(e))
                            )
                        sys.exit(-1)
            except Exception as e:
                logger.exception("Exception in {}:{}".format(self.name, str(e)))
            finally:
                try:
                    self.close_op.run()
                except Exception:
                    pass
                logger.info("{} Exited.".format(self.name))

    def dequeue(self):
        return self.queue.dequeue()


if __name__ == "__main__":
    os.environ["CUDA_VISIBLE_DEVICES"] = ""

    from pose_augment import set_network_input_wh, set_network_scale

    # set_network_input_wh(368, 368)
    set_network_input_wh(480, 320)
    set_network_scale(8)

    # df = get_dataflow('/data/public/rw/coco/annotations', True, '/data/public/rw/coco/')
    df = _get_dataflow_onlyread(
        "/data/public/rw/coco/annotations", True, "/data/public/rw/coco/"
    )
    # df = get_dataflow('/root/coco/annotations', False, img_path='http://gpu-twg.kakaocdn.net/braincloud/COCO/')

    from tensorpack.dataflow.common import TestDataSpeed

    TestDataSpeed(df).start()
    sys.exit(0)

    with tf.Session() as sess:
        df.reset_state()
        t1 = time.time()
        for idx, dp in enumerate(df.get_data()):
            if idx == 0:
                for d in dp:
                    logger.info("%d dp shape={}".format(d.shape))
            print(time.time() - t1)
            t1 = time.time()
            CocoPose.display_image(
                dp[0], dp[1].astype(np.float32), dp[2].astype(np.float32)
            )
            print(dp[1].shape, dp[2].shape)
            pass

    logger.info("done")



import matplotlib as mpl

mpl.use("Agg")  # training mode, no screen should be open. (It will block training loop)

import argparse
import logging
import os
import time

import cv2
import numpy as np
import tensorflow as tf
from tqdm import tqdm

from tf_pose.pose_dataset import get_dataflow_batch, DataFlowToQueue, CocoPose
from tf_pose.pose_augment import set_network_input_wh, set_network_scale
from tf_pose.common import get_sample_images
from tf_pose.networks import get_network

logger = logging.getLogger("train")
logger.handlers.clear()
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Training codes for Openpose using Tensorflow"
    )
    parser.add_argument("--model", default="mobilenet_v2_1.4", help="model name")
    parser.add_argument(
        "--datapath", type=str, default="/data/public/rw/coco/annotations"
    )
    parser.add_argument("--imgpath", type=str, default="/data/public/rw/coco/")
    parser.add_argument("--batchsize", type=int, default=64)
    parser.add_argument("--gpus", type=int, default=4)
    parser.add_argument("--max-epoch", type=int, default=600)
    parser.add_argument("--lr", type=str, default="0.001")
    parser.add_argument("--tag", type=str, default="test")
    parser.add_argument("--checkpoint", type=str, default="")

    parser.add_argument("--input-width", type=int, default=432)
    parser.add_argument("--input-height", type=int, default=368)
    parser.add_argument("--quant-delay", type=int, default=-1)
    args = parser.parse_args()

    modelpath = logpath = "./models/train/"

    if args.gpus <= 0:
        raise Exception("gpus <= 0")

    # define input placeholder
    set_network_input_wh(args.input_width, args.input_height)
    scale = 4

    if args.model in ["cmu", "vgg"] or "mobilenet" in args.model:
        scale = 8

    set_network_scale(scale)
    output_w, output_h = args.input_width // scale, args.input_height // scale

    logger.info("define model+")
    with tf.device(tf.DeviceSpec(device_type="CPU")):
        input_node = tf.placeholder(
            tf.float32,
            shape=(args.batchsize, args.input_height, args.input_width, 3),
            name="image",
        )
        vectmap_node = tf.placeholder(
            tf.float32, shape=(args.batchsize, output_h, output_w, 38), name="vectmap"
        )
        heatmap_node = tf.placeholder(
            tf.float32, shape=(args.batchsize, output_h, output_w, 19), name="heatmap"
        )

        # prepare data
        df = get_dataflow_batch(
            args.datapath, True, args.batchsize, img_path=args.imgpath
        )
        enqueuer = DataFlowToQueue(
            df, [input_node, heatmap_node, vectmap_node], queue_size=100
        )
        q_inp, q_heat, q_vect = enqueuer.dequeue()

    df_valid = get_dataflow_batch(
        args.datapath, False, args.batchsize, img_path=args.imgpath
    )
    df_valid.reset_state()
    validation_cache = []

    val_image = get_sample_images(args.input_width, args.input_height)
    logger.debug("tensorboard val image: %d" % len(val_image))
    logger.debug(q_inp)
    logger.debug(q_heat)
    logger.debug(q_vect)

    # define model for multi-gpu
    q_inp_split, q_heat_split, q_vect_split = (
        tf.split(q_inp, args.gpus),
        tf.split(q_heat, args.gpus),
        tf.split(q_vect, args.gpus),
    )

    output_vectmap = []
    output_heatmap = []
    losses = []
    last_losses_l1 = []
    last_losses_l2 = []
    outputs = []
    for gpu_id in range(args.gpus):
        with tf.device(tf.DeviceSpec(device_type="GPU", device_index=gpu_id)):
            with tf.variable_scope(tf.get_variable_scope(), reuse=(gpu_id > 0)):
                net, pretrain_path, last_layer = get_network(
                    args.model, q_inp_split[gpu_id]
                )
                if args.checkpoint:
                    pretrain_path = args.checkpoint
                vect, heat = net.loss_last()
                output_vectmap.append(vect)
                output_heatmap.append(heat)
                outputs.append(net.get_output())

                l1s, l2s = net.loss_l1_l2()
                for idx, (l1, l2) in enumerate(zip(l1s, l2s)):
                    loss_l1 = tf.nn.l2_loss(
                        tf.concat(l1, axis=0) - q_vect_split[gpu_id],
                        name="loss_l1_stage%d_tower%d" % (idx, gpu_id),
                    )
                    loss_l2 = tf.nn.l2_loss(
                        tf.concat(l2, axis=0) - q_heat_split[gpu_id],
                        name="loss_l2_stage%d_tower%d" % (idx, gpu_id),
                    )
                    losses.append(tf.reduce_mean([loss_l1, loss_l2]))

                last_losses_l1.append(loss_l1)
                last_losses_l2.append(loss_l2)

    outputs = tf.concat(outputs, axis=0)

    with tf.device(tf.DeviceSpec(device_type="GPU")):
        # define loss
        total_loss = tf.reduce_sum(losses) / args.batchsize
        total_loss_ll_paf = tf.reduce_sum(last_losses_l1) / args.batchsize
        total_loss_ll_heat = tf.reduce_sum(last_losses_l2) / args.batchsize
        total_loss_ll = tf.reduce_sum([total_loss_ll_paf, total_loss_ll_heat])

        # define optimizer
        step_per_epoch = 121745 // args.batchsize
        global_step = tf.Variable(0, trainable=False)
        if "," not in args.lr:
            starter_learning_rate = float(args.lr)
            # learning_rate = tf.train.exponential_decay(starter_learning_rate, global_step,
            #                                            decay_steps=10000, decay_rate=0.33, staircase=True)
            learning_rate = tf.train.cosine_decay(
                starter_learning_rate,
                global_step,
                args.max_epoch * step_per_epoch,
                alpha=0.0,
            )
        else:
            lrs = [float(x) for x in args.lr.split(",")]
            boundaries = [step_per_epoch * 5 * i for i, _ in range(len(lrs)) if i > 0]
            learning_rate = tf.train.piecewise_constant(global_step, boundaries, lrs)

    if args.quant_delay >= 0:
        logger.info("train using quantized mode, delay=%d" % args.quant_delay)
        g = tf.get_default_graph()
        tf.contrib.quantize.create_training_graph(
            input_graph=g, quant_delay=args.quant_delay
        )

    # optimizer = tf.train.RMSPropOptimizer(learning_rate, decay=0.0005, momentum=0.9, epsilon=1e-10)
    optimizer = tf.train.AdamOptimizer(learning_rate, epsilon=1e-8)
    # optimizer = tf.train.MomentumOptimizer(learning_rate, momentum=0.8, use_locking=True, use_nesterov=True)
    update_ops = tf.get_collection(tf.GraphKeys.UPDATE_OPS)
    with tf.control_dependencies(update_ops):
        train_op = optimizer.minimize(
            total_loss, global_step, colocate_gradients_with_ops=True
        )
    logger.info("define model-")

    # define summary
    tf.summary.scalar("loss", total_loss)
    tf.summary.scalar("loss_lastlayer", total_loss_ll)
    tf.summary.scalar("loss_lastlayer_paf", total_loss_ll_paf)
    tf.summary.scalar("loss_lastlayer_heat", total_loss_ll_heat)
    tf.summary.scalar("queue_size", enqueuer.size())
    tf.summary.scalar("lr", learning_rate)
    merged_summary_op = tf.summary.merge_all()

    valid_loss = tf.placeholder(tf.float32, shape=[])
    valid_loss_ll = tf.placeholder(tf.float32, shape=[])
    valid_loss_ll_paf = tf.placeholder(tf.float32, shape=[])
    valid_loss_ll_heat = tf.placeholder(tf.float32, shape=[])
    sample_train = tf.placeholder(tf.float32, shape=(4, 640, 640, 3))
    sample_valid = tf.placeholder(tf.float32, shape=(12, 640, 640, 3))
    train_img = tf.summary.image("training sample", sample_train, 4)
    valid_img = tf.summary.image("validation sample", sample_valid, 12)
    valid_loss_t = tf.summary.scalar("loss_valid", valid_loss)
    valid_loss_ll_t = tf.summary.scalar("loss_valid_lastlayer", valid_loss_ll)
    merged_validate_op = tf.summary.merge(
        [train_img, valid_img, valid_loss_t, valid_loss_ll_t]
    )

    saver = tf.train.Saver(max_to_keep=1000)
    config = tf.ConfigProto(allow_soft_placement=True, log_device_placement=False)
    config.gpu_options.allow_growth = True
    with tf.Session(config=config) as sess:
        logger.info("model weights initialization")
        sess.run(tf.global_variables_initializer())

        if args.checkpoint and os.path.isdir(args.checkpoint):
            logger.info("Restore from checkpoint...")
            # loader = tf.train.Saver(net.restorable_variables())
            # loader.restore(sess, tf.train.latest_checkpoint(args.checkpoint))
            saver.restore(sess, tf.train.latest_checkpoint(args.checkpoint))
            logger.info("Restore from checkpoint...Done")
        elif pretrain_path:
            logger.info("Restore pretrained weights... %s" % pretrain_path)
            if ".npy" in pretrain_path:
                net.load(pretrain_path, sess, False)
            else:
                try:
                    loader = tf.train.Saver(
                        net.restorable_variables(only_backbone=False)
                    )
                    loader.restore(sess, pretrain_path)
                except:
                    logger.info("Restore only weights in backbone layers.")
                    loader = tf.train.Saver(net.restorable_variables())
                    loader.restore(sess, pretrain_path)
            logger.info("Restore pretrained weights...Done")

        logger.info("prepare file writer")
        file_writer = tf.summary.FileWriter(os.path.join(logpath, args.tag), sess.graph)

        logger.info("prepare coordinator")
        coord = tf.train.Coordinator()
        enqueuer.set_coordinator(coord)
        enqueuer.start()

        logger.info("Training Started.")
        time_started = time.time()
        last_gs_num = last_gs_num2 = 0
        initial_gs_num = sess.run(global_step)

        last_log_epoch1 = last_log_epoch2 = -1
        while True:
            _, gs_num = sess.run([train_op, global_step])
            curr_epoch = float(gs_num) / step_per_epoch

            if gs_num > step_per_epoch * args.max_epoch:
                break

            if gs_num - last_gs_num >= 500:
                (
                    train_loss,
                    train_loss_ll,
                    train_loss_ll_paf,
                    train_loss_ll_heat,
                    lr_val,
                    summary,
                ) = sess.run(
                    [
                        total_loss,
                        total_loss_ll,
                        total_loss_ll_paf,
                        total_loss_ll_heat,
                        learning_rate,
                        merged_summary_op,
                    ]
                )

                # log of training loss / accuracy
                batch_per_sec = (gs_num - initial_gs_num) / (time.time() - time_started)
                logger.info(
                    "epoch=%.2f step=%d, %0.4f examples/sec lr=%f, loss=%g, loss_ll=%g, loss_ll_paf=%g, loss_ll_heat=%g"
                    % (
                        gs_num / step_per_epoch,
                        gs_num,
                        batch_per_sec * args.batchsize,
                        lr_val,
                        train_loss,
                        train_loss_ll,
                        train_loss_ll_paf,
                        train_loss_ll_heat,
                    )
                )
                last_gs_num = gs_num

                if last_log_epoch1 < curr_epoch:
                    file_writer.add_summary(summary, curr_epoch)
                    last_log_epoch1 = curr_epoch

            if gs_num - last_gs_num2 >= 2000:
                # save weights
                saver.save(
                    sess,
                    os.path.join(modelpath, args.tag, "model_latest"),
                    global_step=global_step,
                )

                average_loss = (
                    average_loss_ll
                ) = average_loss_ll_paf = average_loss_ll_heat = 0
                total_cnt = 0

                if len(validation_cache) == 0:
                    for images_test, heatmaps, vectmaps in tqdm(df_valid.get_data()):
                        validation_cache.append((images_test, heatmaps, vectmaps))
                    df_valid.reset_state()
                    del df_valid
                    df_valid = None

                # log of test accuracy
                for images_test, heatmaps, vectmaps in validation_cache:
                    (
                        lss,
                        lss_ll,
                        lss_ll_paf,
                        lss_ll_heat,
                        vectmap_sample,
                        heatmap_sample,
                    ) = sess.run(
                        [
                            total_loss,
                            total_loss_ll,
                            total_loss_ll_paf,
                            total_loss_ll_heat,
                            output_vectmap,
                            output_heatmap,
                        ],
                        feed_dict={
                            q_inp: images_test,
                            q_vect: vectmaps,
                            q_heat: heatmaps,
                        },
                    )
                    average_loss += lss * len(images_test)
                    average_loss_ll += lss_ll * len(images_test)
                    average_loss_ll_paf += lss_ll_paf * len(images_test)
                    average_loss_ll_heat += lss_ll_heat * len(images_test)
                    total_cnt += len(images_test)

                logger.info(
                    "validation(%d) %s loss=%f, loss_ll=%f, loss_ll_paf=%f, loss_ll_heat=%f"
                    % (
                        total_cnt,
                        args.tag,
                        average_loss / total_cnt,
                        average_loss_ll / total_cnt,
                        average_loss_ll_paf / total_cnt,
                        average_loss_ll_heat / total_cnt,
                    )
                )
                last_gs_num2 = gs_num

                sample_image = [enqueuer.last_dp[0][i] for i in range(4)]
                outputMat = sess.run(
                    outputs,
                    feed_dict={
                        q_inp: np.array(
                            (sample_image + val_image) * max(1, (args.batchsize // 16))
                        )
                    },
                )
                pafMat, heatMat = outputMat[:, :, :, 19:], outputMat[:, :, :, :19]

                sample_results = []
                for i in range(len(sample_image)):
                    test_result = CocoPose.display_image(
                        sample_image[i], heatMat[i], pafMat[i], as_numpy=True
                    )
                    test_result = cv2.resize(test_result, (640, 640))
                    test_result = test_result.reshape([640, 640, 3]).astype(float)
                    sample_results.append(test_result)

                test_results = []
                for i in range(len(val_image)):
                    test_result = CocoPose.display_image(
                        val_image[i],
                        heatMat[len(sample_image) + i],
                        pafMat[len(sample_image) + i],
                        as_numpy=True,
                    )
                    test_result = cv2.resize(test_result, (640, 640))
                    test_result = test_result.reshape([640, 640, 3]).astype(float)
                    test_results.append(test_result)

                # save summary
                summary = sess.run(
                    merged_validate_op,
                    feed_dict={
                        valid_loss: average_loss / total_cnt,
                        valid_loss_ll: average_loss_ll / total_cnt,
                        valid_loss_ll_paf: average_loss_ll_paf / total_cnt,
                        valid_loss_ll_heat: average_loss_ll_heat / total_cnt,
                        sample_valid: test_results,
                        sample_train: sample_results,
                    },
                )
                if last_log_epoch2 < curr_epoch:
                    file_writer.add_summary(summary, curr_epoch)
                    last_log_epoch2 = curr_epoch

        saver.save(
            sess, os.path.join(modelpath, args.tag, "model"), global_step=global_step
        )
    logger.info("optimization finished. %f" % (time.time() - time_started))



from __future__ import absolute_import

import tensorflow as tf

from tf_pose import network_base


class MobilenetNetwork(network_base.BaseNetwork):
    def __init__(self, inputs, trainable=True, conv_width=1.0, conv_width2=None):
        self.conv_width = conv_width
        self.conv_width2 = conv_width2 if conv_width2 else conv_width
        self.num_refine = 4
        network_base.BaseNetwork.__init__(self, inputs, trainable)

    def setup(self):
        min_depth = 8
        depth = lambda d: max(int(d * self.conv_width), min_depth)
        depth2 = lambda d: max(int(d * self.conv_width2), min_depth)

        with tf.variable_scope(None, "MobilenetV1"):
            (
                self.feed("image")
                .convb(3, 3, depth(32), 2, name="Conv2d_0")
                .separable_conv(3, 3, depth(64), 1, name="Conv2d_1")
                .separable_conv(3, 3, depth(128), 2, name="Conv2d_2")
                .separable_conv(3, 3, depth(128), 1, name="Conv2d_3")
                .separable_conv(3, 3, depth(256), 2, name="Conv2d_4")
                .separable_conv(3, 3, depth(256), 1, name="Conv2d_5")
                .separable_conv(3, 3, depth(512), 1, name="Conv2d_6")
                .separable_conv(3, 3, depth(512), 1, name="Conv2d_7")
                .separable_conv(3, 3, depth(512), 1, name="Conv2d_8")
                # .separable_conv(3, 3, depth(512), 1, name='Conv2d_9')
                # .separable_conv(3, 3, depth(512), 1, name='Conv2d_10')
                # .separable_conv(3, 3, depth(512), 1, name='Conv2d_11')
                # .separable_conv(3, 3, depth(1024), 2, name='Conv2d_12')
                # .separable_conv(3, 3, depth(1024), 1, name='Conv2d_13')
            )

        (self.feed("Conv2d_1").max_pool(2, 2, 2, 2, name="Conv2d_1_pool"))
        (self.feed("Conv2d_7").upsample(2, name="Conv2d_7_upsample"))

        (
            self.feed("Conv2d_1_pool", "Conv2d_3", "Conv2d_7_upsample").concat(
                3, name="feat_concat"
            )
        )

        feature_lv = "feat_concat"
        with tf.variable_scope(None, "Openpose"):
            prefix = "MConv_Stage1"
            (
                self.feed(feature_lv)
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_1")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_2")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_3")
                .separable_conv(1, 1, depth2(512), 1, name=prefix + "_L1_4")
                .separable_conv(1, 1, 38, 1, relu=False, name=prefix + "_L1_5")
            )

            (
                self.feed(feature_lv)
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_1")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_2")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_3")
                .separable_conv(1, 1, depth2(512), 1, name=prefix + "_L2_4")
                .separable_conv(1, 1, 19, 1, relu=False, name=prefix + "_L2_5")
            )

            for stage_id in range(self.num_refine):
                prefix_prev = "MConv_Stage%d" % (stage_id + 1)
                prefix = "MConv_Stage%d" % (stage_id + 2)
                (
                    self.feed(prefix_prev + "_L1_5", prefix_prev + "_L2_5", feature_lv)
                    .concat(3, name=prefix + "_concat")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L1_1")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L1_2")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L1_3")
                    .separable_conv(1, 1, depth2(128), 1, name=prefix + "_L1_4")
                    .separable_conv(1, 1, 38, 1, relu=False, name=prefix + "_L1_5")
                )

                (
                    self.feed(prefix + "_concat")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L2_1")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L2_2")
                    .separable_conv(7, 7, depth2(128), 1, name=prefix + "_L2_3")
                    .separable_conv(1, 1, depth2(128), 1, name=prefix + "_L2_4")
                    .separable_conv(1, 1, 19, 1, relu=False, name=prefix + "_L2_5")
                )

            # final result
            (
                self.feed(
                    "MConv_Stage%d_L2_5" % self.get_refine_num(),
                    "MConv_Stage%d_L1_5" % self.get_refine_num(),
                ).concat(3, name="concat_stage7")
            )

    def loss_l1_l2(self):
        l1s = []
        l2s = []
        for layer_name in sorted(self.layers.keys()):
            if "_L1_5" in layer_name:
                l1s.append(self.layers[layer_name])
            if "_L2_5" in layer_name:
                l2s.append(self.layers[layer_name])

        return l1s, l2s

    def loss_last(self):
        return (
            self.get_output("MConv_Stage%d_L1_5" % self.get_refine_num()),
            self.get_output("MConv_Stage%d_L2_5" % self.get_refine_num()),
        )

    def restorable_variables(self):
        vs = {
            v.op.name: v
            for v in tf.global_variables()
            if "MobilenetV1/Conv2d" in v.op.name
            and "RMSProp" not in v.op.name
            and "Momentum" not in v.op.name
            and "Ada" not in v.op.name
        }
        return vs

    def get_refine_num(self):
        return self.num_refine + 1



from enum import Enum

import tensorflow as tf
import cv2


regularizer_conv = 0.004
regularizer_dsconv = 0.0004
batchnorm_fused = True
activation_fn = tf.nn.relu


class CocoPart(Enum):
    Nose = 0
    Neck = 1
    RShoulder = 2
    RElbow = 3
    RWrist = 4
    LShoulder = 5
    LElbow = 6
    LWrist = 7
    RHip = 8
    RKnee = 9
    RAnkle = 10
    LHip = 11
    LKnee = 12
    LAnkle = 13
    REye = 14
    LEye = 15
    REar = 16
    LEar = 17
    Background = 18


class MPIIPart(Enum):
    RAnkle = 0
    RKnee = 1
    RHip = 2
    LHip = 3
    LKnee = 4
    LAnkle = 5
    RWrist = 6
    RElbow = 7
    RShoulder = 8
    LShoulder = 9
    LElbow = 10
    LWrist = 11
    Neck = 12
    Head = 13

    @staticmethod
    def from_coco(human):
        # t = {
        #     MPIIPart.RAnkle: CocoPart.RAnkle,
        #     MPIIPart.RKnee: CocoPart.RKnee,
        #     MPIIPart.RHip: CocoPart.RHip,
        #     MPIIPart.LHip: CocoPart.LHip,
        #     MPIIPart.LKnee: CocoPart.LKnee,
        #     MPIIPart.LAnkle: CocoPart.LAnkle,
        #     MPIIPart.RWrist: CocoPart.RWrist,
        #     MPIIPart.RElbow: CocoPart.RElbow,
        #     MPIIPart.RShoulder: CocoPart.RShoulder,
        #     MPIIPart.LShoulder: CocoPart.LShoulder,
        #     MPIIPart.LElbow: CocoPart.LElbow,
        #     MPIIPart.LWrist: CocoPart.LWrist,
        #     MPIIPart.Neck: CocoPart.Neck,
        #     MPIIPart.Nose: CocoPart.Nose,
        # }

        t = [
            (MPIIPart.Head, CocoPart.Nose),
            (MPIIPart.Neck, CocoPart.Neck),
            (MPIIPart.RShoulder, CocoPart.RShoulder),
            (MPIIPart.RElbow, CocoPart.RElbow),
            (MPIIPart.RWrist, CocoPart.RWrist),
            (MPIIPart.LShoulder, CocoPart.LShoulder),
            (MPIIPart.LElbow, CocoPart.LElbow),
            (MPIIPart.LWrist, CocoPart.LWrist),
            (MPIIPart.RHip, CocoPart.RHip),
            (MPIIPart.RKnee, CocoPart.RKnee),
            (MPIIPart.RAnkle, CocoPart.RAnkle),
            (MPIIPart.LHip, CocoPart.LHip),
            (MPIIPart.LKnee, CocoPart.LKnee),
            (MPIIPart.LAnkle, CocoPart.LAnkle),
        ]

        pose_2d_mpii = []
        visibilty = []
        for mpi, coco in t:
            if coco.value not in human.body_parts.keys():
                pose_2d_mpii.append((0, 0))
                visibilty.append(False)
                continue
            pose_2d_mpii.append(
                (human.body_parts[coco.value].x, human.body_parts[coco.value].y)
            )
            visibilty.append(True)
        return pose_2d_mpii, visibilty


CocoPairs = [
    (1, 2),
    (1, 5),
    (2, 3),
    (3, 4),
    (5, 6),
    (6, 7),
    (1, 8),
    (8, 9),
    (9, 10),
    (1, 11),
    (11, 12),
    (12, 13),
    (1, 0),
    (0, 14),
    (14, 16),
    (0, 15),
    (15, 17),
    (2, 16),
    (5, 17),
]  # = 19
CocoPairsRender = CocoPairs[:-2]
# CocoPairsNetwork = [
#     (12, 13), (20, 21), (14, 15), (16, 17), (22, 23), (24, 25), (0, 1), (2, 3), (4, 5),
#     (6, 7), (8, 9), (10, 11), (28, 29), (30, 31), (34, 35), (32, 33), (36, 37), (18, 19), (26, 27)
#  ]  # = 19

CocoColors = [
    [255, 0, 0],
    [255, 85, 0],
    [255, 170, 0],
    [255, 255, 0],
    [170, 255, 0],
    [85, 255, 0],
    [0, 255, 0],
    [0, 255, 85],
    [0, 255, 170],
    [0, 255, 255],
    [0, 170, 255],
    [0, 85, 255],
    [0, 0, 255],
    [85, 0, 255],
    [170, 0, 255],
    [255, 0, 255],
    [255, 0, 170],
    [255, 0, 85],
]


def read_imgfile(path, width=None, height=None):
    val_image = cv2.imread(path, cv2.IMREAD_COLOR)
    if width is not None and height is not None:
        val_image = cv2.resize(val_image, (width, height))
    return val_image


def get_sample_images(w, h):
    val_image = [
        read_imgfile("./images/p1.jpg", w, h),
        read_imgfile("./images/p2.jpg", w, h),
        read_imgfile("./images/p3.jpg", w, h),
        read_imgfile("./images/golf.jpg", w, h),
        read_imgfile("./images/hand1.jpg", w, h),
        read_imgfile("./images/hand2.jpg", w, h),
        read_imgfile("./images/apink1_crop.jpg", w, h),
        read_imgfile("./images/ski.jpg", w, h),
        read_imgfile("./images/apink2.jpg", w, h),
        read_imgfile("./images/apink3.jpg", w, h),
        read_imgfile("./images/handsup1.jpg", w, h),
        read_imgfile("./images/p3_dance.png", w, h),
    ]
    return val_image


def to_str(s):
    if not isinstance(s, str):
        return s.decode("utf-8")
    return s



from __future__ import absolute_import

import tensorflow as tf

from tf_pose import network_base
from tf_pose.mobilenet import mobilenet_v2
from tf_pose.network_base import layer


class Mobilenetv2Network(network_base.BaseNetwork):
    def __init__(self, inputs, trainable=True, conv_width=1.0, conv_width2=1.0):
        self.conv_width = conv_width
        self.refine_width = conv_width2
        network_base.BaseNetwork.__init__(self, inputs, trainable)

    @layer
    def base(self, input, name):
        with tf.contrib.slim.arg_scope(mobilenet_v2.training_scope()):
            net, endpoints = mobilenet_v2.mobilenet_base(
                input,
                self.conv_width,
                finegrain_classification_mode=(self.conv_width < 1.0),
            )
            for k, tensor in sorted(list(endpoints.items()), key=lambda x: x[0]):
                self.layers["%s/%s" % (name, k)] = tensor
                # print(k, tensor.shape)
            return net

    def setup(self):
        depth2 = lambda x: int(x * self.refine_width)

        self.feed("image").base(name="base")

        # TODO : add more feature with downsample?
        # self.feed('base/layer_4/output').max_pool(2, 2, 2, 2, name='base/layer_4/output/downsample')
        # self.feed('base/layer_4/output').avg_pool(2, 2, 2, 2, name='base/layer_4/output/downsample')
        self.feed("base/layer_14/output").upsample(
            factor="base/layer_7/output", name="base/layer_14/output/upsample"
        )
        (
            self.feed(
                "base/layer_7/output",
                "base/layer_14/output/upsample",
                # 'base/layer_4/output/downsample'
            ).concat(3, name="feat_concat")
        )

        feature_lv = "feat_concat"
        with tf.variable_scope(None, "Openpose"):
            prefix = "MConv_Stage1"
            (
                self.feed(feature_lv)
                # .se_block(name=prefix + '_L1_se', ratio=8)
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_1")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_2")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_3")
                .separable_conv(1, 1, depth2(512), 1, name=prefix + "_L1_4")
                .separable_conv(1, 1, 38, 1, relu=False, name=prefix + "_L1_5")
            )

            (
                self.feed(feature_lv)
                # .se_block(name=prefix + '_L2_se', ratio=8)
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_1")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_2")
                .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_3")
                .separable_conv(1, 1, depth2(512), 1, name=prefix + "_L2_4")
                .separable_conv(1, 1, 19, 1, relu=False, name=prefix + "_L2_5")
            )

            for stage_id in range(5):
                prefix_prev = "MConv_Stage%d" % (stage_id + 1)
                prefix = "MConv_Stage%d" % (stage_id + 2)
                (
                    self.feed(prefix_prev + "_L1_5", prefix_prev + "_L2_5", feature_lv)
                    .concat(3, name=prefix + "_concat")
                    # .se_block(name=prefix + '_L1_se', ratio=8)
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_1")
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_2")
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L1_3")
                    .separable_conv(1, 1, depth2(128), 1, name=prefix + "_L1_4")
                    .separable_conv(1, 1, 38, 1, relu=False, name=prefix + "_L1_5")
                )

                (
                    self.feed(prefix + "_concat")
                    # .se_block(name=prefix + '_L2_se', ratio=8)
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_1")
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_2")
                    .separable_conv(3, 3, depth2(128), 1, name=prefix + "_L2_3")
                    .separable_conv(1, 1, depth2(128), 1, name=prefix + "_L2_4")
                    .separable_conv(1, 1, 19, 1, relu=False, name=prefix + "_L2_5")
                )

            # final result
            (
                self.feed("MConv_Stage6_L2_5", "MConv_Stage6_L1_5").concat(
                    3, name="concat_stage7"
                )
            )

    def loss_l1_l2(self):
        l1s = []
        l2s = []
        for layer_name in sorted(self.layers.keys()):
            if "_L1_5" in layer_name:
                l1s.append(self.layers[layer_name])
            if "_L2_5" in layer_name:
                l2s.append(self.layers[layer_name])

        return l1s, l2s

    def loss_last(self):
        return (
            self.get_output("MConv_Stage6_L1_5"),
            self.get_output("MConv_Stage6_L2_5"),
        )

    def restorable_variables(self, only_backbone=True):
        vs = {
            v.op.name: v
            for v in tf.global_variables()
            if (
                "MobilenetV2" in v.op.name
                or (only_backbone is False and "Openpose" in v.op.name)
            )
            and
            # 'global_step' not in v.op.name and
            # 'beta1_power' not in v.op.name and 'beta2_power' not in v.op.name and
            "quant" not in v.op.name
            and "RMSProp" not in v.op.name
            and "Momentum" not in v.op.name
            and "Ada" not in v.op.name
            and "Adam" not in v.op.name
        }
        # print(set([v.op.name for v in tf.global_variables()]) - set(list(vs.keys())))
        return vs



from __future__ import absolute_import

from tf_pose import network_base


class DSConvNetwork(network_base.BaseNetwork):
    def __init__(self, inputs, trainable=True, conv_width=1.0):
        self.conv_width = conv_width
        network_base.BaseNetwork.__init__(self, inputs, trainable)

    def setup(self):
        (
            self.feed("image")
            .conv(3, 3, 64, 1, 1, name="conv1_1", trainable=False)
            # .conv(3, 3, 64, 1, 1, name='conv1_2', trainable=True)     # TODO
            .separable_conv(3, 3, round(self.conv_width * 64), 2, name="conv1_2")
            # .max_pool(2, 2, 2, 2, name='pool1_stage1')
            .separable_conv(3, 3, round(self.conv_width * 128), 1, name="conv2_1")
            .separable_conv(3, 3, round(self.conv_width * 128), 2, name="conv2_2")
            # .max_pool(2, 2, 2, 2, name='pool2_stage1')
            .separable_conv(3, 3, round(self.conv_width * 256), 1, name="conv3_1")
            .separable_conv(3, 3, round(self.conv_width * 256), 1, name="conv3_2")
            .separable_conv(3, 3, round(self.conv_width * 256), 1, name="conv3_3")
            .separable_conv(3, 3, round(self.conv_width * 256), 2, name="conv3_4")
            # .max_pool(2, 2, 2, 2, name='pool3_stage1')
            .separable_conv(3, 3, round(self.conv_width * 512), 1, name="conv4_1")
            .separable_conv(3, 3, round(self.conv_width * 512), 1, name="conv4_2")
            .separable_conv(3, 3, round(self.conv_width * 256), 1, name="conv4_3_CPM")
            .separable_conv(3, 3, 128, 1, name="conv4_4_CPM")
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_1_CPM_L1"
            )
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_2_CPM_L1"
            )
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_3_CPM_L1"
            )
            .conv(1, 1, 512, 1, 1, name="conv5_4_CPM_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="conv5_5_CPM_L1")
        )

        (
            self.feed("conv4_4_CPM")
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_1_CPM_L2"
            )
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_2_CPM_L2"
            )
            .separable_conv(
                3, 3, round(self.conv_width * 128), 1, name="conv5_3_CPM_L2"
            )
            .conv(1, 1, 512, 1, 1, name="conv5_4_CPM_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="conv5_5_CPM_L2")
        )

        (
            self.feed("conv5_5_CPM_L1", "conv5_5_CPM_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage2")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage2_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage2_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage2_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage2_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage2_L1"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage2_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage2_L1")
        )

        (
            self.feed("concat_stage2")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage2_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage2_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage2_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage2_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage2_L2"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage2_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage2_L2")
        )

        (
            self.feed("Mconv7_stage2_L1", "Mconv7_stage2_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage3")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage3_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage3_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage3_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage3_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage3_L1"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage3_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage3_L1")
        )

        (
            self.feed("concat_stage3")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage3_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage3_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage3_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage3_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage3_L2"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage3_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage3_L2")
        )

        (
            self.feed("Mconv7_stage3_L1", "Mconv7_stage3_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage4")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage4_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage4_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage4_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage4_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage4_L1"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage4_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage4_L1")
        )

        (
            self.feed("concat_stage4")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage4_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage4_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage4_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage4_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage4_L2"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage4_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage4_L2")
        )

        (
            self.feed("Mconv7_stage4_L1", "Mconv7_stage4_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage5")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage5_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage5_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage5_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage5_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage5_L1"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage5_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage5_L1")
        )

        (
            self.feed("concat_stage5")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage5_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage5_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage5_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage5_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage5_L2"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage5_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage5_L2")
        )

        (
            self.feed("Mconv7_stage5_L1", "Mconv7_stage5_L2", "conv4_4_CPM")
            .concat(3, name="concat_stage6")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage6_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage6_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage6_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage6_L1"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage6_L1"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage6_L1")
            .conv(1, 1, 38, 1, 1, relu=False, name="Mconv7_stage6_L1")
        )

        (
            self.feed("concat_stage6")
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv1_stage6_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv2_stage6_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv3_stage6_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv4_stage6_L2"
            )
            .separable_conv(
                7, 7, round(self.conv_width * 128), 1, name="Mconv5_stage6_L2"
            )
            .conv(1, 1, 128, 1, 1, name="Mconv6_stage6_L2")
            .conv(1, 1, 19, 1, 1, relu=False, name="Mconv7_stage6_L2")
        )

        (
            self.feed("Mconv7_stage6_L2", "Mconv7_stage6_L1").concat(
                3, name="concat_stage7"
            )
        )



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import subprocess
import setuptools
from setuptools import dist
from distutils.core import setup, Extension

dist.Distribution().fetch_build_eggs(["Cython", "numpy"])
import numpy as np

cwd = os.path.dirname(os.path.abspath(__file__))
subprocess.check_output(["bash", "models/graph/cmu/download.sh"], cwd=cwd)
POSE_DIR = os.path.realpath(os.path.dirname(__file__))

REQUIRED_PACKAGES = [
    "argparse>=1.1",
    "dill==0.2.7.1",
    "fire >= 0.1.3",
    "matplotlib >= 2.2.2",
    "psutil >= 5.4.5",
    "requests >= 2.18.4",
    "scikit-image >= 0.13.1",
    "scipy >= 1.1.0",
    "slidingwindow >= 0.0.13",
    "opencv-python",
    "tqdm >= 4.23.4",
    "tensorflow < 2.0",
    "tensorpack >= 0.8.5",
    "pycocotools",
]

DEPENDENCY_LINKS = [
    "git+https://github.com/ppwwyyxx/tensorpack.git#egg=tensorpack",
]

EXT = Extension(
    "_pafprocess",
    sources=[
        "tf_pose/pafprocess/pafprocess_wrap.cpp",
        "tf_pose/pafprocess/pafprocess.cpp",
    ],
    swig_opts=["-c++"],
    include_dirs=[np.get_include()],
)

setuptools.setup(
    name="tf_pose",
    version="0.5.0",
    description="Deep Pose Estimation implemented using Tensorflow with Custom Architectures for fast inference.",
    long_description="This is a fork of the origninal https://github.com/ildoonet/tf-pose-estimation. Main change is "
                     "to restrict tensorflow 1.X. There are some incompatibility issue with tensorflow 2+ we haven't "
                     "sorted out",
    long_description_content_type="text/markdown",
    install_requires=REQUIRED_PACKAGES,
    dependency_links=DEPENDENCY_LINKS,
    url="https://github.com/ildoonet/tf-pose-estimation/",
    author="Ildoo Kim",
    author_email="ildoo@ildoo.net",
    license="Apache License 2.0",
    package_dir={"tf_pose_data": "models"},
    packages=["tf_pose_data"]
    + [
        pkg_name
        for pkg_name in setuptools.find_packages()  # main package
        if "tf_pose" in pkg_name
    ],
    ext_modules=[EXT],
    package_data={
        "tf_pose_data": ["graph/cmu/graph_opt.pb", "graph/mobilenet_thin/graph_opt.pb"]
    },
    py_modules=["pafprocess"],
    zip_safe=False,
)



import math


class DimOrder(object):
    """
	Represents the order of the dimensions in a dataset's shape.
	"""

    ChannelHeightWidth = ["c", "h", "w"]
    HeightWidthChannel = ["h", "w", "c"]


class SlidingWindow(object):
    """
	Represents a single window into a larger dataset.
	"""

    def __init__(self, x, y, w, h, dimOrder, transform=None):
        """
		Creates a new window with the specified dimensions and transform
		"""
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.dimOrder = dimOrder
        self.transform = transform

    def apply(self, matrix):
        """
		Slices the supplied matrix and applies any transform bound to this window
		"""
        view = matrix[self.indices()]
        return self.transform(view) if self.transform != None else view

    def getRect(self):
        """
		Returns the window bounds as a tuple of (x,y,w,h)
		"""
        return (self.x, self.y, self.w, self.h)

    def setRect(self, rect):
        """
		Sets the window bounds from a tuple of (x,y,w,h)
		"""
        self.x, self.y, self.w, self.h = rect

    def indices(self, includeChannel=True):
        """
		Retrieves the indices for this window as a tuple of slices
		"""
        if self.dimOrder == DimOrder.HeightWidthChannel:

            # Equivalent to [self.y:self.y+self.h+1, self.x:self.x+self.w+1]
            return (slice(self.y, self.y + self.h), slice(self.x, self.x + self.w))

        elif self.dimOrder == DimOrder.ChannelHeightWidth:

            if includeChannel is True:

                # Equivalent to [:, self.y:self.y+self.h+1, self.x:self.x+self.w+1]
                return (
                    slice(None, None),
                    slice(self.y, self.y + self.h),
                    slice(self.x, self.x + self.w),
                )

            else:

                # Equivalent to [self.y:self.y+self.h+1, self.x:self.x+self.w+1]
                return (slice(self.y, self.y + self.h), slice(self.x, self.x + self.w))

        else:
            raise Error("Unsupported order of dimensions: " + str(self.dimOrder))

    def __str__(self):
        return (
            "("
            + str(self.x)
            + ","
            + str(self.y)
            + ","
            + str(self.w)
            + ","
            + str(self.h)
            + ")"
        )

    def __repr__(self):
        return self.__str__()


def generate(
    data, dimOrder, maxWindowSizeW, maxWindowSizeH, overlapPercent, transforms=[]
):
    """
	Generates a set of sliding windows for the specified dataset.
	"""

    # Determine the dimensions of the input data
    width = data.shape[dimOrder.index("w")]
    height = data.shape[dimOrder.index("h")]

    # Generate the windows
    return generateForSize(
        width,
        height,
        dimOrder,
        maxWindowSizeW,
        maxWindowSizeH,
        overlapPercent,
        transforms,
    )


def generateForSize(
    width,
    height,
    dimOrder,
    maxWindowSizeW,
    maxWindowSizeH,
    overlapPercent,
    transforms=[],
):
    """
	Generates a set of sliding windows for a dataset with the specified dimensions and order.
	"""

    # If the input data is smaller than the specified window size,
    # clip the window size to the input size on both dimensions
    windowSizeX = min(maxWindowSizeW, width)
    windowSizeY = min(maxWindowSizeH, height)

    # Compute the window overlap and step size
    windowOverlapX = int(math.floor(windowSizeX * overlapPercent))
    windowOverlapY = int(math.floor(windowSizeY * overlapPercent))
    stepSizeX = windowSizeX - windowOverlapX
    stepSizeY = windowSizeY - windowOverlapY

    # Determine how many windows we will need in order to cover the input data
    lastX = width - windowSizeX
    lastY = height - windowSizeY
    xOffsets = list(range(0, lastX + 1, stepSizeX))
    yOffsets = list(range(0, lastY + 1, stepSizeY))

    # Unless the input data dimensions are exact multiples of the step size,
    # we will need one additional row and column of windows to get 100% coverage
    if len(xOffsets) == 0 or xOffsets[-1] != lastX:
        xOffsets.append(lastX)
    if len(yOffsets) == 0 or yOffsets[-1] != lastY:
        yOffsets.append(lastY)

    # Generate the list of windows
    windows = []
    for xOffset in xOffsets:
        for yOffset in yOffsets:
            for transform in [None] + transforms:
                windows.append(
                    SlidingWindow(
                        x=xOffset,
                        y=yOffset,
                        w=windowSizeX,
                        h=windowSizeY,
                        dimOrder=dimOrder,
                        transform=transform,
                    )
                )

    return windows



import base64
import os

import cv2
from functools import lru_cache

from tf_pose import common
from tf_pose import eval
from tf_pose.estimator import TfPoseEstimator
from tf_pose.networks import get_graph_path, model_wh

Estimator = TfPoseEstimator


@lru_cache(maxsize=1)
def get_estimator(model="cmu", resize="0x0"):
    w, h = model_wh(resize)
    if w == 0 or h == 0:
        e = TfPoseEstimator(get_graph_path(model), target_size=(432, 368))
    else:
        e = TfPoseEstimator(get_graph_path(model), target_size=(w, h))

    return e


def infer(image, model="cmu", resize="0x0", resize_out_ratio=4.0):
    """

    :param image:
    :param model:
    :param resize:
    :param resize_out_ratio:
    :return: coco_style_keypoints array
    """
    w, h = model_wh(resize)
    e = get_estimator(model, resize)

    # estimate human poses from a single image !
    image = common.read_imgfile(image, None, None)
    if image is None:
        raise Exception("Image can not be read, path=%s" % image)
    humans = e.inference(
        image, resize_to_default=(w > 0 and h > 0), upsample_size=resize_out_ratio
    )
    image_h, image_w = image.shape[:2]

    if "TERM_PROGRAM" in os.environ and "iTerm" in os.environ["TERM_PROGRAM"]:
        image = TfPoseEstimator.draw_humans(image, humans, imgcopy=False)
        image_str = cv2.imencode(".jpg", image)[1].tostring()
        print(
            "\033]1337;File=name=;inline=1:"
            + base64.b64encode(image_str).decode("utf-8")
            + "\a"
        )

    return [
        (eval.write_coco_json(human, image_w, image_h), human.score) for human in humans
    ]
